/*

   Copyright 2016 Toby Kenney

   This file is part of Adequate Bootstrap.

   Adequate Bootstrap is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Adequate Bootstrap is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Adequate Bootstrap.  If not, see <http://www.gnu.org/licenses/>.

 */

#include <iostream>
#include <fstream>
#include <cmath>
#include <boost/math/distributions/normal.hpp>
#include <boost/math/distributions/chi_squared.hpp>
#include <cstdlib>
#include <ctime>

#include <boost/math/distributions/binomial.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/variate_generator.hpp>
#include <boost/generator_iterator.hpp>
#include <boost/random/binomial_distribution.hpp>
#include <boost/random/mersenne_twister.hpp>
#include <boost/random/normal_distribution.hpp>

boost::math::normal stdnorm(0.0, 1.0);

#define BSSIZE 10000
//Number of bootstraps to perform.


using namespace std;

extern long double chisqcval;


typedef long double (*NP_estimator)(unsigned int n,long double *data,unsigned int *samp);

NP_estimator NPest=NULL;
NP_estimator CILest=NULL;
NP_estimator CIUest=NULL;


void MLEProb(unsigned int n,const unsigned int *outcomes,long double *p){
//Assumes probability is a monotone function of the xvalues. Calculates
//the MLE of the probability for each x value under this constraint.

//outcomes is a matrix whose first column is number of
//         successes and whose last column is number of trials.
//         Rows are sorted in increasing order of xvalue.

  //  long double *minfor=new long double[n];
  long double *sucint=new long double[n*n];
  long double *totint=new long double[n*n];
  for(unsigned int i=0;i<n;i++){
    *(sucint+i*n+i)=*(outcomes+2*i);
    *(totint+i*n+i)=*(outcomes+2*i+1);
    for(unsigned int j=i+1;j<n;j++){
      *(sucint+i*n+j)=*(sucint+i*n+j-1)+*(outcomes+2*j);
      *(totint+i*n+j)=*(totint+i*n+j-1)+*(outcomes+2*j+1);
    };
  };
  long double max=0;

  for(unsigned int i=0;i<n;i++){
    *(p+i)=1;
    for(unsigned int j=i;j<n;j++){
      long double rat=((long double)*(sucint+i*n+j))/((long double)*(totint+i*n+j));
      if(rat<*(p+i)){
	*(p+i)=rat;
      };
    };
    if(*(p+i)<max){
      *(p+i)=max;
    }else{
      max=*(p+i);
    };
  };
  delete[] sucint;
  delete[] totint;
};


inline unsigned int position(long double prob,unsigned int n,const long double *phat){
  unsigned int N=n-1;
  unsigned int Nl=0;
  if(*(phat+N)>prob){
    if(*(phat)<prob){
      while(N-Nl>1){
	unsigned int Nm=(N+Nl)/2;
	if(*(phat+Nm)<prob){
	  Nl=Nm;
	}else{
	  N=Nm;
	};
      };
    }else{
      N=0;
    };
  };
  return N;
};

ofstream outlog;

void write_to_log(unsigned int n,long double *phat,unsigned int *outcomes ,unsigned int *xvals){
  if(outlog.is_open()){
    for(unsigned int i=0;i<n;i++){
      outlog<<*(xvals+i)<<" "<<*(phat+i)<<" "<<*(outcomes+2*i)<<"/"<<*(outcomes+2*i+1)<<"\n";
    };
  };
};


unsigned int solveprob(unsigned int n,unsigned int *xvals,char(*trial)(unsigned int x,const void *data),long double prob,const void *trialdata=NULL,unsigned int mxwd=100,unsigned int mxdp=10000){
//xvals are the values to be attempted
//trial is a function which produces a single run for one x value.
//prob is the target probability.
  unsigned int *outcomes=new unsigned int[2*n];
  for(unsigned int i=0;i<n;i++){       
    *(outcomes+2*i+1)=1;
    *(outcomes+2*i)=0;
    //Try to get more accurate measurement.
    for(unsigned int j=0;j<1;j++){   
      if((*trial)(*(xvals+i),trialdata)){
	(*(outcomes+2*i))++;
      };
    };
  };
  //  write_to_log(n,outcomes,xvals);
  long double *phat=new long double[n];
  MLEProb(n,outcomes,phat);
  unsigned int N=position(prob,n,phat);
//Now produce more results close to the value N
  for(unsigned int k=1;k<11;k++){
    unsigned int intwidth=(mxwd/k);
    unsigned int intdepth=(mxdp/intwidth);
    for(unsigned int i=(N>intwidth)?(N-intwidth):0;i<N+intwidth&&i<n;i++){
      *(outcomes+2*i+1)+=intdepth;
      for(unsigned int j=0;j<intdepth;j++){
	if(trial(xvals[i],trialdata)){
	  (*(outcomes+2*i))++;
	};
      };
    };
    MLEProb(n,outcomes,phat);
    N=position(prob,n,phat);
    //    write_to_log(n,phat,outcomes,xvals);
    //    ofstream outlog("outcomes.log",ios_base::app);
    //    if(outlog.is_open()){
    //      for(unsigned int i=0;i<n;i++){
    //	outlog<<*(xvals+i)<<" "<<*(outcomes+2*i)<<" "<<*(outcomes+2*i)<<"\n";
    //      };
    //    };
  };
  //  write_to_log(n,phat,outcomes,xvals);
  delete[] outcomes;
  delete[] phat;
  return N;
};

unsigned int solveprobfast(unsigned int n,unsigned int *xvals,char(*trial)(unsigned int x,const void *data),long double prob,const void *trialdata=NULL){
  unsigned int numstep=log10(n)-1;
  unsigned int firststep=pow(10,numstep);
  //First run on every hundredth size
  unsigned int nh=n/firststep+1;
  unsigned int *xvh=new unsigned int[nh];
  for(unsigned int i=0;i<nh;i++){
    *(xvh+i)=*(xvals+i*firststep);
  };
  unsigned int N=solveprob(nh,xvh,trial,prob,trialdata,20,1000);

  unsigned int bt=0;


  unsigned int increasedepth=0;
  if(numstep>1){
      increasedepth=3000/(numstep-1);
  };

  for(;numstep>0;numstep--){
    unsigned int tp=n-1;
    if((N+10)*firststep<tp){
      tp=(N+10)*firststep;
    };
    bt=0;
    if(N>10){
      bt=(N-10)*firststep;
    };
    firststep/=10;

    unsigned int nh2=(tp-bt+1)/firststep;
    if(nh2>nh){
      delete[] xvh;
      xvh=new unsigned int[nh2];
      nh=nh2;
    };
    for(unsigned int i=0;i<nh2;i++){
      *(xvh+i)=*(xvals+bt+i*firststep);
    };
    N=solveprob(nh2,xvh,trial,prob,trialdata,40,5000-increasedepth*(numstep-1));
  };
  delete[]xvh;
  return(*(xvals+N+bt));

//  tp=n-1;
//  if((N+10)*10+bt<tp){
//    tp=(N+10)*10+bt;
//  };
//  if(N*10+bt>100){
//    bt=(N-10)*10+bt;
//  };
//  unsigned int nh3=tp-bt+1;
//  if(nh<nh3&&nh2<nh3){
//    delete[] xvh;
//    xvh=new unsigned int[nh3];
//  };
//  for(unsigned int i=0;i<nh3;i++){
//    *(xvh+i)=*(xvals+bt+i);
//  };  
//  N=solveprob(nh3,xvh,trial,prob,trialdata,40,5000);
};

#define NUMBOOTSTRAP 10000

inline void subsample(unsigned int n,const long double *data,unsigned int x,long double *samp){
  unsigned int rmx=n*(RAND_MAX/n);
  for(unsigned int i=0;i<x;i++){
    unsigned int rnd=rand();
    while(rnd>rmx){
      rnd=rand(); //redraw if too large.
    };
    *(samp+i)=*(data+rnd%n);
  };
};

const long double qchisq95[]={3.841459,  5.991465,  7.814728,  9.487729, 11.070498, 12.591587, 14.067140,  15.507313, 16.918978, 18.307038, 19.675138, 21.026070, 22.362032, 23.684791, 24.995790, 26.296228, 27.587112, 28.869299, 30.143527, 31.410433};






char fivepercent(unsigned int x,const void *data){
  return(rand()>(RAND_MAX*0.95));
};

void getpos(unsigned int n,const long double *list,unsigned int numps,const unsigned int *pos,long double *to){
  //pos should be sorted.

  long double *newlist=new long double[n];
  unsigned int nl=0;
  unsigned int nu=1;
  long double pivot=*list;
  for(unsigned int i=1;i<n;i++){
    if(*(list+i)<pivot){
      *(newlist+nl++)=*(list+i);
    }else{
      *(newlist+n-nu++)=*(list+i);
    };
  };
  *(newlist+nl)=pivot;
  unsigned int i;
  for(i=0;i<numps&&pos[i]<nl;i++);
  if(i>0){
    getpos(nl,newlist,i,pos,to);
  };
  if(i<numps){
    if(pos[i]==nl){
      *(to+i)=pivot;    
      if(i+1<numps){
	unsigned int *newpos=new unsigned int[numps-i-1];
	for(unsigned int j=0;j<numps-i-1;j++){
	  newpos[j]=pos[i+j+1]-nl-1;
	};
	getpos(nu-1,newlist+nl+1,numps-i-1,newpos,to+i+1);
	delete[] newpos;
      };
    }else{
      //pos[i]>nl
      unsigned int *newpos=new unsigned int[numps-i];
      for(unsigned int j=0;j<numps-i;j++){
	newpos[j]=pos[i+j]-nl-1;
      };
      getpos(nu-1,newlist+nl+1,numps-i,newpos,to+i);
      delete[] newpos;
    };    
  };
  delete[] newlist;
};


void testMLEprob(){
  unsigned int *xvals=new unsigned int[19971];
  for(unsigned int i=0;i<19971;i++){
    *(xvals+i)=i+30;
  };
  void *data=NULL;
  for(unsigned int i=0;i<1000;i++){
    unsigned int bssize=solveprobfast(19971,xvals,fivepercent,0.5,&data);
    cout<<bssize;
    if((i+1)%10){
      cout<<" ";
    }else{
      cout<<"\n";
    };
  };
};


int compareLongDouble(const void *p1,const void *p2){
  if(*((long double *)p1)<*((long double *)p2)){
    return -1;
  }else if(*((long double *)p1)>*((long double *)p2)){
    return 1;
  }else{
    return 0;
  };
};

class dblll{
  long double val;
  dblll *next;
public:
  dblll(long double v){val=v;next=NULL;};
  dblll(long double v,dblll *nxt){val=v;next=nxt;};
  void insert(long double v){dblll *ins=new dblll(val,next);next=ins;val=v;};
  ~dblll(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(long double *to){*to=val;if(next!=NULL){next->write(to+1);};};
};


class dblctll{
  long double val;
  unsigned int freq;
  dblctll *next;
public:
  dblctll(long double v,unsigned int i){val=v;freq=i;next=NULL;};
  dblctll(long double v,unsigned int i,dblctll *nxt){val=v;freq=i;next=nxt;};
  void insert(long double v,unsigned int i){dblctll *ins=new dblctll(val,freq,next);next=ins;val=v;freq=i;};
  ~dblctll(){if(next){delete(next);};};
  unsigned int length(unsigned int ans=0){ans++;if(next==NULL){return(ans);}else{return(next->length(ans));};};
  void write(long double *to,unsigned int *frq){*to=val;*frq=freq;if(next!=NULL){next->write(to+1,frq+1);};};
};



long double VaR_level=0.01;

class AdequateBootstrapIntervals{
public:
  unsigned int absz;
  long double abl;
  long double abu;
  long double sbl;
  long double sbu;
  long double nbl;
  long double nbu;
  long double nl;
  long double nu;
};


void rand_split(const long double *data,long double *train,long double *test,unsigned int numtrain,unsigned int numtest){
  unsigned int tefull=0;
  unsigned int trfull=0;
  for(;(trfull<numtrain)&&(tefull<numtest);){
    if((((long double) rand())/RAND_MAX)<numtrain/(numtrain+numtest)){
      *(train+trfull)=*(data+trfull+tefull);
      trfull++;
    }else{
      *(test+tefull)=*(data+trfull+tefull);
      tefull++;
    };
  };
  if(trfull==numtrain){
    for(;tefull<numtest;tefull++){
      *(test+tefull)=*(data+numtrain+tefull);
    };
  }else{
    for(;trfull<numtest;tefull++){
      *(train+trfull)=*(data+numtest+trfull);
    };
  };
};

#include "distributions.h"


#include "tests.h"

//3 modes of operation:
//1st mode: data analysis



unsigned int solveprob(unsigned int n,unsigned int *xvals,AdequacyTest trial,long double prob,Distribution& dist,const void *trialdata=NULL,unsigned int mxwd=100,unsigned int mxdp=10000){
//xvals are the values to be attempted
//trial is a function which produces a single run for one x value.
//prob is the target probability.
  unsigned int *outcomes=new unsigned int[2*n];
  for(unsigned int i=0;i<n;i++){       
    *(outcomes+2*i+1)=1;
    *(outcomes+2*i)=0;
    //Try to get more accurate measurement.
    for(unsigned int j=0;j<1;j++){   
      //      dist.print();
      if((*trial)(*(xvals+i),trialdata,dist)){
	(*(outcomes+2*i))++;
      };
    };
  };
  //  write_to_log(n,outcomes,xvals);
  long double *phat=new long double[n];
  MLEProb(n,outcomes,phat);
  unsigned int N=position(prob,n,phat);
//Now produce more results close to the value N
  for(unsigned int k=1;k<11;k++){
    unsigned int intwidth=(mxwd/k);
    unsigned int intdepth=(mxdp/intwidth);

    for(unsigned int i=(N>intwidth)?(N-intwidth):0;i<N+intwidth&&i<n;i++){
      *(outcomes+2*i+1)+=intdepth;
      for(unsigned int j=0;j<intdepth;j++){
	if(trial(xvals[i],trialdata,dist)){
	  (*(outcomes+2*i))++;
	};
      };
    };
    MLEProb(n,outcomes,phat);
    N=position(prob,n,phat);
    //    write_to_log(n,phat,outcomes,xvals);
    //    ofstream outlog("outcomes.log",ios_base::app);
    //    if(outlog.is_open()){
    //      for(unsigned int i=0;i<n;i++){
    //	outlog<<*(xvals+i)<<" "<<*(outcomes+2*i)<<" "<<*(outcomes+2*i)<<"\n";
    //      };
    //    };
  };
  write_to_log(n,phat,outcomes,xvals);
  delete[] outcomes;
  delete[] phat;
  return N;
};

unsigned int solveprobfast(unsigned int n,unsigned int *xvals,AdequacyTest trial,long double prob,Distribution& dist,const void *trialdata=NULL){
  unsigned int numstep=log10(n)-1;
  //  cout<<"numstep="<<numstep<<"\n";
  unsigned int firststep=pow(10,numstep);
  //First run on every hundredth size
  unsigned int nh=(n-1)/firststep+1;
  unsigned int *xvh=new unsigned int[nh];
  for(unsigned int i=0;i<nh;i++){
    *(xvh+i)=*(xvals+i*firststep);
  };
  unsigned int N=solveprob(nh,xvh,trial,prob,dist,trialdata,20,1000);
  //    cout<<N<<"\n";

  unsigned int bt=0;

  //  cout<<n<<" "<<numstep<<" "<<firststep<<"\n";

  unsigned int increasedepth=0;
  if(numstep>1){
      increasedepth=3000/(numstep-1);
  };


  for(;numstep>0;numstep--){
    unsigned int tp=n-1;
    if((N+10)*firststep<tp){
      tp=(N+10)*firststep;
    };
    bt=0;
    if(N>10){
      bt=(N-10)*firststep;
    };
    firststep/=10;
    //    cout<<n<<" "<<numstep<<" "<<firststep<<"\n";
    //    cout<<bt<<" "<<tp<<"\n";
    unsigned int nh2=(tp-bt+1)/firststep;
    if(nh2>nh){
      delete[] xvh;
      xvh=new unsigned int[nh2];
      nh=nh2;
    };
    for(unsigned int i=0;i<nh2;i++){
      *(xvh+i)=*(xvals+bt+i*firststep);
    };
    N=solveprob(nh2,xvh,trial,prob,dist,trialdata,40,5000-increasedepth*(numstep-1));
    N+=bt/firststep;
    //    cout<<N<<"\n";
  };
  delete[]xvh;
  //  cout<<N<<" "<<bt<<"\n";
  return(*(xvals+N));


  //First run on every hundredth size

//
//  unsigned int nh=n/100;
//  unsigned int N=0;
//  unsigned int *xvh=NULL;
//  if(nh>0){
//    xvh=new unsigned int[nh];
//    for(unsigned int i=0;i<nh;i++){
//      *(xvh+i)=*(xvals+i*100);
//    };
//    N=solveprob(nh,xvh,trial,prob,dist,trialdata,20,1000);
//  };
//  //  cout<<N<<"\n";
//  unsigned int tp=n-1;
//  if((N+10)*100<tp){
//    tp=(N+10)*100;
//  };
//  unsigned int bt=0;
//  if(N>10){
//    bt=(N-10)*100;
//  };
//  unsigned int nh2=(tp-bt)/10;
//  if(nh2>nh){
//    if(xvh!=NULL){
//      delete[] xvh;
//    };
//    xvh=new unsigned int[nh2];
//  };
//  for(unsigned int i=0;i<nh2;i++){
//    *(xvh+i)=*(xvals+bt+i*10);
//  };
//  N=solveprob(nh2,xvh,trial,prob,dist,trialdata,40,2000);
//  //  cout<<N<<"\n";
//  tp=n-1;
//  if((N+10)*10+bt<tp){
//    tp=(N+10)*10+bt;
//  };
//  if(N*10+bt>100){
//    bt=(N-10)*10+bt;
//  };
//  unsigned int nh3=tp-bt+1;
//  if(nh<nh3&&nh2<nh3){
//    delete[] xvh;
//    xvh=new unsigned int[nh3];
//  };
//  for(unsigned int i=0;i<nh3;i++){
//    *(xvh+i)=*(xvals+bt+i);
//  };  
//  N=solveprob(nh3,xvh,trial,prob,dist,trialdata,40,5000);
//  delete[]xvh;
//  return(*(xvals+N+bt));
};

long double np_mean(unsigned int n,long double *data,unsigned int *samp){
  long double sum=0;
  unsigned int tot=0;
  for(unsigned int i=0;i<n;i++){
    sum+=*(data+i)*(*(samp+i));
    tot+=*(samp+i);
  };
  return sum/tot;
};


unsigned int solveprobscan(unsigned int n,unsigned int *xvals,AdequacyTest trial,long double prob,Distribution& dist,const void *trialdata=NULL){
  //Quickly checks whether the range is sensible for the values needed.
  if(n<100){
    return solveprob(n,xvals,trial,prob,dist,trialdata,15,20);
  };
  unsigned int N=solveprob(100,xvals,trial,prob,dist,trialdata,15,20);
  if(N>95){
    return solveprobfast(n,xvals,trial,prob,dist,trialdata);
  };  
  if(N>80){
    if(n>199){
      N=solveprobfast(200,xvals,trial,prob,dist,trialdata);
      if(N<170){
	return N;
      };
    };
    return solveprobfast(n,xvals,trial,prob,dist,trialdata);
  };
  if(N>5){
    return solveprobfast(100,xvals,trial,prob,dist,trialdata);
  };
  return N;
};


long double np_sd(unsigned int n,long double *data,unsigned int *samp){
  long double sum=0;
  long double sumsq=0;
  unsigned int tot=0;
  for(unsigned int i=0;i<n;i++){
    sum+=*(data+i)*(*(samp+i));
    sumsq+=*(data+i)*(*(data+i))*(*(samp+i));
    tot+=*(samp+i);
  };  
  return sqrt(sumsq/tot-sum*sum/(tot*tot));
};

long double np_VaR_at=0.05;

long double np_VaR(unsigned int n,long double *data,unsigned int *samp){
  unsigned int tot=0;
  for(unsigned int i=0;i<n;i++){
    tot+=*(samp+i);
  };  
  unsigned int pos=tot*np_VaR_at;
  unsigned int i=0;
  unsigned int cumtot=0;
  for(i=0;cumtot<pos;cumtot+=*(samp+i++));
  //  cout<<tot<<" "<<pos<<" "<<i<<" "<<cumtot<<" "<<*(data+i)<<"\n";
  return *(data+i); //Should do interpolation??
};

long double np_Gini(unsigned int n,long double *data,unsigned int *samp){
  unsigned int cumtot=0;
  long double U2=0;
  long double U1=0;
  for(unsigned int i=0;i<n;cumtot+=*(samp+i++)){
    U1+=*(samp+i)*(*(data+i));
    U2+=((*(samp+i))*cumtot+(*(samp+i)*(*(samp+i)-1))/2)*(*(data+i));
  };
  //  cout<<tot<<" "<<pos<<" "<<i<<" "<<cumtot<<" "<<*(data+i)<<"\n";
  return 2*U2/U1/(cumtot-1)-1;
};



long double cil_VaR_at=0.05;
long double ciu_VaR_at=0.05;




ofstream savedata;

unsigned int jackknife=0;

unsigned int frequency_table=0;

unsigned int grouped_data=0;

unsigned int ngroup=0;

unsigned int maxbssize=0;


void read_option(const char* const optname){
    char *at=NULL;
    char *at2=NULL;
    char *eq=NULL;
    char* name=new char[strlen(optname)+1];
    strcpy(name,optname);
    unsigned int len=strlen(name);
    for(unsigned int i=0;i<len;i++){
      if(*(name+i)=='@'){
	*(name+i)='\0';
	if(at==NULL){
	  at=(name+i+1);
	}else{
	  at2=(name+i+1);
	};
      }else if(*(name+i)=='='){
	*(name+i)='\0';
	eq=(name+i+1);	
      };
    };
    if(!strcmp(name,"seed")){
      if(at!=NULL){
	cout<<"@ option not used for seed option. Ignoring.\n";
      };
      if(eq==NULL){
	cout<<"Seed value unspecified. Ignoring seed setting.\n";
      }else{	
	unsigned int val=atoi(eq);
	cout<<"Seeding random generator with value "<<val<<".\n";
	srand(val);
      };
    }else if(!strcmp(name,"NP")){
      if(eq==NULL){
	cout<<"NP method unspecified. Ignoring NP setting.\n";
      }else{	
	if(!strcmp(eq,"mean")){
	  NPest=np_mean;
	}else if(!strcmp(eq,"sd")){
	  NPest=np_sd;
	}else if(!strcmp(eq,"VaR")){
	  long double atval=0.05;
	  if(at!=NULL){
	    atval=atof(at);
	  };
	  np_VaR_at=atval;
	  NPest=np_VaR;
	}else if(!strcmp(eq,"Gini")){
	  NPest=np_Gini;
	}else{
	  cout<<"Nonparametric estimator \""<<eq<<"\" not currently supported. Ignoring option.\n";
	};
      };
    }else if(!strcmp(name,"CI")){
      if(eq==NULL){
	cout<<"CI method unspecified. Ignoring CI setting.\n";
      }else{	
	if(*eq!='['){
	  cout<<"first method: \""<<eq<<"\"\n";
	  cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	}else{
	  eq++;
	  char* eq2=eq;	
	  for(;*eq2!='\0'&&*eq2!=',';eq2++);
	  if(*eq2!=','){
	    if(++eq2==at){
	      for(;*eq2!='\0'&&*eq2!=',';eq2++);
	      if(*eq2!=','){
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<at2<<"\"\n";
		cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	      };
	    }else{
		cout<<"Arguments:\t\""<<eq<<"\"\n\t\t\""<<at<<"\"\n\t\t\""<<eq2<<"\"\n\t\t\""<<at2<<"\"\n";	      cout<<"CI format \"CI=[LOWER,UPPER]\".\n";
	    };
	  };
	  *(eq2++)='\0';
	  if(!strcmp(eq,"mean")){
	    CILest=np_mean;
	  }else if(!strcmp(eq,"sd")){
	    CILest=np_sd;
	  }else if(!strcmp(eq,"VaR")){
	    long double atval=0.05;
	    if(at!=NULL){
	      atval=atof(at);
	    };
	    cil_VaR_at=atval;
	    CILest=np_VaR;
	  }else{
	    cout<<"Estimator \""<<eq<<"\" not currently supported. Ignoring option.\n";
	    return;
	  };
	  if(!strcmp(eq2,"mean")){
	    CIUest=np_mean;
	  }else if(!strcmp(eq2,"sd")){
	    CIUest=np_sd;
	  }else if(!strcmp(eq2,"VaR")){
	    long double atval=0.05;
	    if(at2!=NULL){
	      atval=atof(at2);
	    };
	    ciu_VaR_at=atval;
	    CIUest=np_VaR;
	  }else{
	    cout<<"Estimator \""<<eq2<<"\" not currently supported. Ignoring option.\n";
	    CILest=NULL;
	  };
	};
      };
    }else if(!strcmp(name,"jackknife")){
      if(eq==NULL){
	jackknife=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  jackknife=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  jackknife=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for jackknife.\n";
	};
      };      
    }else if(!strcmp(name,"log")){
      if(eq==NULL){
	outlog.open("outcomes.log");
	if(!outlog.is_open()){
	  cout<<"Problem opening default log file \"outcomes.log\"\n";
	};
      }else{
	outlog.open(eq);
	if(!outlog.is_open()){
	  cout<<"Problem opening log file \""<<eq<<"\"\n";
	};	
      };      
    }else if(!strcmp(name,"save")){
      if(eq==NULL){
	eq=(char *)"saved_data";
	//No checking for overwriting file;
      };
      savedata.open(eq);      
      if(!savedata.is_open()){
	cout<<"Unable to open file \""<<eq<<"\".\n";
      };
    }else if(!strcmp(name,"frequency")){
      if(eq==NULL){
	frequency_table=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	  frequency_table=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  frequency_table=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for frequency.\n";
	};
      };
    }else if(!strcmp(name,"grouped")){
      if(eq==NULL){
	grouped_data=1;
      }else{
	if(!strcmp(eq,"1")||!strcmp(eq,"T")||!strcmp(eq,"yes")||!strcmp(eq,"true")||!strcmp(eq,"True")||!strcmp(eq,"Yes")){
	 grouped_data=1;
	}else if(!strcmp(eq,"0")||!strcmp(eq,"F")||!strcmp(eq,"no")||!strcmp(eq,"No")||!strcmp(eq,"False")||!strcmp(eq,"false")){
	  grouped_data=0;
	}else{
	  cout<<"Could not determine meaning of setting \""<<eq<<"\" for grouped data.\n";
	};
      };
    }else if(!strcmp(name,"maxbssize")){
      if(eq!=NULL){
	maxbssize=atoi(eq);
      };
    }else{
      cout<<"Option \""<<name<<"\" not currently supported. Ignoring option.\n";
      // long double atval=0;
      // if(at!=NULL){
      // 	atval=atof(at);
      // 	cout<<atval<<"\n";
      // };
    };

    delete[] name;
};





void analyse_data(Distribution &truedist,AdequacyTest adequacy_test,const Data &A){
  //Analyse Data
  truedist.print();

  unsigned int *xvals=new unsigned int[A.n-19];
  for(unsigned int i=0;i<A.n-19;i++){
    *(xvals+i)=i+20;
  };
  unsigned int bssize=solveprobfast(A.n-19,xvals,adequacy_test,0.5,truedist,&A);

  delete[] xvals;

  AdequateBootstrapIntervals ans;

  //  cout<<"Adequate Bootstrap size: "<<bssize<<".\n";
  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];
  long double *npbs=new long double[BSSIZE];

  long double *absamp=new long double[bssize];
  long double *bsamp=new long double[A.n];
  unsigned int *sbsmp=new unsigned int[A.n];

  if(jackknife){
    unsigned int jks=A.n/2;
    if(bssize>jks){
      cout<<"Adequate Jackknife size reduced from "<<bssize<<" to "<<jks<<".\n";
      bssize=jks;
    };
    for(unsigned int i=0;i<BSSIZE;i++){
      sortedjackknife(A.n,bssize,A.data,absamp);
      sortedjackknife(A.n,jks,A.data,bsamp);    
      sortedsubsample(A.n,A.n,sbsmp);
      //Still bootstrap for non-parametric estimator.
      truedist.estimate_data(bssize,absamp);
      *(bs+i)=truedist.parval();
      truedist.estimate_data(jks,bsamp);
      *(stdbs+i)=truedist.parval();
      if(NPest!=NULL){
	*(npbs+i)=(*NPest)(A.n,A.data,sbsmp);
      };
    };
  }else{
    for(unsigned int i=0;i<BSSIZE;i++){
      subsample(A.n,A.data,bssize,absamp);
      subsample(A.n,A.data,A.n,bsamp);    
      sortedsubsample(A.n,A.n,sbsmp);
      truedist.estimate_data(bssize,absamp);
      *(bs+i)=truedist.parval();
      truedist.estimate_data(A.n,bsamp);
      *(stdbs+i)=truedist.parval();
      if(NPest!=NULL){
	*(npbs+i)=(*NPest)(A.n,A.data,sbsmp);
      };
    };
  };
  unsigned int pos[2]={0.025*BSSIZE,0.975*BSSIZE};

  long double ci[2];
  long double stdci[2];
  long double npci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};

  //  cout<<target<<" "<<confprob[0]<<" "<<confprob[1]<<" ";

  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  //  cout<<posl<<" "<<posu<<"\n";
  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  if(NPest!=NULL){
    getpos(BSSIZE,npbs,2,pos,npci);
    ans.nbl=npci[0];
    ans.nbu=npci[1];
  };

  if(CILest!=NULL&&CIUest!=NULL){
    for(unsigned int i=0;i<A.n;i++){
      *(sbsmp+i)=1;
    };
    np_VaR_at=cil_VaR_at;
    ans.nl=(*CILest)(A.n,A.data,sbsmp);
    np_VaR_at=ciu_VaR_at;
    ans.nu=(*CIUest)(A.n,A.data,sbsmp);
    //    cout<<cil_VaR_at<<" "<<ciu_VaR_at;
  };


  delete[] absamp;
  delete[] bsamp;
  delete[] sbsmp;
  delete[] bs;
  delete[] stdbs;
  delete[] npbs;


  cout<<"\nAdequate Bootstrap size: "<<ans.absz<<"\n";

  cout<<"Adequate Bootstrap interval: ["<<ans.abl<<", "<<ans.abu<<"]\n";
  cout<<"Standard Bootstrap interval: ["<<ans.sbl<<", "<<ans.sbu<<"]\n";

  if(NPest!=NULL){
    cout<<"Nonparametric Bootstrap interval: ["<<truedist.untransform_data(ans.nbl)<<", "<<truedist.untransform_data(ans.nbu)<<"]\n";
  };

  if(CILest!=NULL&&CIUest!=NULL){
    cout<<"Theoretical Confidence interval: ["<<truedist.untransform_data(ans.nl)<<", "<<truedist.untransform_data(ans.nu)<<"]\n";
  };  
};

void analyse_data_frequency(Distribution &truedist,AdequacyTest adequacy_test,const FreqTable &A){
  //Analyse Data
  truedist.print();

  unsigned int tot=0;
  for(unsigned int i=0;i<A.n;i++){
    tot+=*(A.count+i);
  };
  //If tot is very large trying all values at once could be bad.

  unsigned int bssize=0;
  if(tot<1000000){
    unsigned int *xvals=new unsigned int[tot-19];
    for(unsigned int i=0;i<tot-19;i++){
      *(xvals+i)=i+20;
    };
    bssize=solveprobfast(tot-19,xvals,adequacy_test,0.5,truedist,&A);
    delete[] xvals;
  }else{

    unsigned int step1=sqrt(tot)/10;
    unsigned int *xvals=new unsigned int[(tot-19)/step1];
    for(unsigned int i=0;i<(tot-19)/step1;i++){
      *(xvals+i)=i*step1+20;
    };
    //    cout<<tot<<" "<<step1<<" "<<(tot-19)/step1<<"\n";
    bssize=solveprobscan((tot-19)/step1,xvals,adequacy_test,0.5,truedist,&A);
    //    bssize=solveprobfast((tot-19)/step1,xvals,adequacy_test,0.5,truedist,&A);
    unsigned int lower=20;
    if(bssize>20*step1){
      lower=bssize-20*step1;
    };
    unsigned int upper=tot;
    if(bssize+20*step1<tot){
      upper=bssize+20*step1;
    };
    delete[] xvals;
    xvals=new unsigned int[upper+1-lower];
    for(unsigned int i=0;lower+i<=upper;i++){
      *(xvals+i)=i+lower;
    };
    bssize=solveprobfast(upper+1-lower,xvals,adequacy_test,0.5,truedist,&A);
    delete[] xvals;

  };


  AdequateBootstrapIntervals ans;

  //  cout<<"Adequate Bootstrap size: "<<bssize<<".\n";
  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];
  long double *npbs=new long double[BSSIZE];


  if(jackknife){
    unsigned int *absamp=new unsigned int[A.n];
    unsigned int *bsamp=new unsigned int[A.n];
    unsigned int *sbsmp=new unsigned int[A.n];
    unsigned int jks=tot/2;
    if(bssize>jks){
      cout<<"Adequate Jackknife size reduced from "<<bssize<<" to "<<jks<<".\n";
      bssize=jks;
    };
    for(unsigned int i=0;i<BSSIZE;i++){
      sortedjackknife(A.n,A.count,bssize,absamp);
      sortedjackknife(A.n,A.count,jks,bsamp);    
      sortedsubsample(A.n,A.n,sbsmp);
      //Still bootstrap for non-parametric estimator.
      truedist.estimate_data(A.n,A.data,absamp);
      *(bs+i)=truedist.parval();
      truedist.estimate_data(A.n,A.data,bsamp);
      *(stdbs+i)=truedist.parval();
      if(NPest!=NULL){
	*(npbs+i)=(*NPest)(A.n,A.data,sbsmp);
      };
    };
    delete[] absamp;
    delete[] bsamp;
    delete[] sbsmp;
  }else{
    unsigned int *absamp=new unsigned int[A.n];
    unsigned int *bsamp=new unsigned int[A.n];
    unsigned int *sbsmp=new unsigned int[A.n];
    for(unsigned int i=0;i<BSSIZE;i++){
      sortedsubsample(A.n,A.count,bssize,absamp);
      sortedsubsample(A.n,A.count,tot,bsamp);    
      sortedsubsample(A.n,A.count,tot,sbsmp);
      truedist.estimate_data(A.n,A.data,absamp);
      *(bs+i)=truedist.parval();
      truedist.estimate_data(A.n,A.data,bsamp);
      *(stdbs+i)=truedist.parval();
      if(NPest!=NULL){
	*(npbs+i)=(*NPest)(A.n,A.data,sbsmp);
      };
    };
    delete[] absamp;
    delete[] bsamp;
    delete[] sbsmp;
  };
  unsigned int pos[2]={0.025*BSSIZE,0.975*BSSIZE};

  long double ci[2];
  long double stdci[2];
  long double npci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};

  //  cout<<target<<" "<<confprob[0]<<" "<<confprob[1]<<" ";

  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  //  cout<<posl<<" "<<posu<<"\n";
  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  if(NPest!=NULL){
    getpos(BSSIZE,npbs,2,pos,npci);
    ans.nbl=npci[0];
    ans.nbu=npci[1];
  };

  if(CILest!=NULL&&CIUest!=NULL){
    np_VaR_at=cil_VaR_at;
    ans.nl=(*CILest)(A.n,A.data,A.count);
    np_VaR_at=ciu_VaR_at;
    ans.nu=(*CIUest)(A.n,A.data,A.count);
    //    cout<<cil_VaR_at<<" "<<ciu_VaR_at;
  };


  delete[] bs;
  delete[] stdbs;
  delete[] npbs;


  cout<<"\nAdequate Bootstrap size: "<<ans.absz<<"\n";

  cout<<"Adequate Bootstrap interval: ["<<ans.abl<<", "<<ans.abu<<"]\n";
  cout<<"Standard Bootstrap interval: ["<<ans.sbl<<", "<<ans.sbu<<"]\n";

  if(NPest!=NULL){
    cout<<"Nonparametric Bootstrap interval: ["<<truedist.untransform_data(ans.nbl)<<", "<<truedist.untransform_data(ans.nbu)<<"]\n";
  };

  if(CILest!=NULL&&CIUest!=NULL){
    cout<<"Theoretical Confidence interval: ["<<truedist.untransform_data(ans.nl)<<", "<<truedist.untransform_data(ans.nu)<<"]\n";
  };  
};


void analyse_data_grouped(Distribution &truedist,AdequacyTest adequacy_test,const GroupedData &A){
  //Analyse Data
  truedist.print();


  unsigned int tot=0;
  for(unsigned int i=0;i<A.n;i++){
    tot+=*(A.count+i);
    cout<<(i?*(A.data+i-1):0)<<"-"<<*(A.data+i)<<": "<<*(A.count+i)<<"\n";
  };
  //If tot is very large trying all values at once could be bad.


  cout<<tot<<"\n";

  unsigned int bssize=0;
  if(maxbssize==0){
    if(tot<1000000){
      unsigned int *xvals=new unsigned int[tot-19];
      for(unsigned int i=0;i<tot-19;i++){
	*(xvals+i)=i+20;
      };
      bssize=solveprobfast(tot-19,xvals,adequacy_test,0.5,truedist,&A);
      delete[] xvals;
    }else{
      
      unsigned int step1=sqrt(tot)/10;
      unsigned int *xvals=new unsigned int[(tot-19)/step1];
      for(unsigned int i=0;i<(tot-19)/step1;i++){
	*(xvals+i)=i*step1+20;
      };
      //    cout<<tot<<" "<<step1<<" "<<(tot-19)/step1<<"\n";
      bssize=solveprobscan((tot-19)/step1,xvals,adequacy_test,0.5,truedist,&A);
      //    bssize=solveprobfast((tot-19)/step1,xvals,adequacy_test,0.5,truedist,&A);
      unsigned int lower=20;
      if(bssize>20*step1){
	lower=bssize-20*step1;
      };
      unsigned int upper=tot;
      if(bssize+20*step1<tot){
	upper=bssize+20*step1;
      };
      delete[] xvals;
      xvals=new unsigned int[upper+1-lower];
      for(unsigned int i=0;lower+i<=upper;i++){
	*(xvals+i)=i+lower;
      };
      bssize=solveprobfast(upper+1-lower,xvals,adequacy_test,0.5,truedist,&A);
      delete[] xvals;
    };
  }else{
      unsigned int *xvals=new unsigned int[maxbssize-19];
      for(unsigned int i=0;i<maxbssize-19;i++){
	*(xvals+i)=i+20;
      };
      bssize=solveprobfast(maxbssize-19,xvals,adequacy_test,0.5,truedist,&A);
      delete[] xvals;
  };

  AdequateBootstrapIntervals ans;

  //  cout<<"Adequate Bootstrap size: "<<bssize<<".\n";
  ans.absz=bssize;


  long double *bs=new long double[BSSIZE];
  long double *stdbs=new long double[BSSIZE];
  long double *npbs=new long double[BSSIZE];

  unsigned int *absamp=new unsigned int[A.n];
  unsigned int *bsamp=new unsigned int[A.n];
  unsigned int *sbsmp=new unsigned int[A.n];

  if(jackknife){
    cout<<"Jackknife not yet supported for grouped data.\n";
    exit(0);
  }else{
    for(unsigned int i=0;i<BSSIZE;i++){
      sortedsubsample(A.n,A.count,bssize,absamp);
      sortedsubsample(A.n,A.count,tot,bsamp);    
      sortedsubsample(A.n,A.count,tot,sbsmp);
      //      for(unsigned int j=0;j<A.n;j++){
      //	cout<<*(absamp+j)<<" ";
      //      };
      truedist.estimate_data(A.n,absamp,A.data);
      *(bs+i)=truedist.parval();
      //      cout<<*(bs+i)<<" ";
      truedist.estimate_data(A.n,bsamp,A.data);
      *(stdbs+i)=truedist.parval();
      //      cout<<*(stdbs+i)<<"\n";
      if(NPest!=NULL){
	//	*(npbs+i)=(*NPest)(A.n,A.data,sbsmp);
      };
    };
  };
  unsigned int pos[2]={0.025*BSSIZE,0.975*BSSIZE};

  long double ci[2];
  long double stdci[2];
  long double npci[2];

  //  long double confprob[2]={0.025*exp(target),0.975*exp(target)};

  //  cout<<target<<" "<<confprob[0]<<" "<<confprob[1]<<" ";

  getpos(BSSIZE,bs,2,pos,ci);
  getpos(BSSIZE,stdbs,2,pos,stdci);

  //  cout<<posl<<" "<<posu<<"\n";
  
  ans.abl=ci[0];
  ans.abu=ci[1];
  ans.sbl=stdci[0];
  ans.sbu=stdci[1];


  //Initialise to avoid unnecessary compiler warnings.
  ans.nbl=0;
  ans.nbu=0;
  ans.nl=0;
  ans.nu=0;


  if(NPest!=NULL){
    getpos(BSSIZE,npbs,2,pos,npci);
    ans.nbl=npci[0];
    ans.nbu=npci[1];
  };

  if(CILest!=NULL&&CIUest!=NULL){
    for(unsigned int i=0;i<A.n;i++){
      *(sbsmp+i)=1;
    };
    np_VaR_at=cil_VaR_at;
    ans.nl=(*CILest)(A.n,A.data,sbsmp);
    np_VaR_at=ciu_VaR_at;
    ans.nu=(*CIUest)(A.n,A.data,sbsmp);
    //    cout<<cil_VaR_at<<" "<<ciu_VaR_at;
  };


  delete[] absamp;
  delete[] bsamp;
  delete[] sbsmp;
  delete[] bs;
  delete[] stdbs;
  delete[] npbs;


  cout<<"\nAdequate Bootstrap size: "<<ans.absz<<"\n";

  cout<<"Adequate Bootstrap interval: ["<<ans.abl<<", "<<ans.abu<<"]\n";
  cout<<"Standard Bootstrap interval: ["<<ans.sbl<<", "<<ans.sbu<<"]\n";

  if(NPest!=NULL){
    cout<<"Nonparametric Bootstrap interval: ["<<truedist.untransform_data(ans.nbl)<<", "<<truedist.untransform_data(ans.nbu)<<"]\n";
  };

  if(CILest!=NULL&&CIUest!=NULL){
    cout<<"Theoretical Confidence interval: ["<<truedist.untransform_data(ans.nl)<<", "<<truedist.untransform_data(ans.nu)<<"]\n";
  };  
};

inline long double rnorm(){
  return quantile(stdnorm,((long double)rand())/RAND_MAX);
};

long double chisqlookup(unsigned int n,long double atval){
  //  calculates the critical value of a chi-square distribution with
  //  n degrees of freedom at significance level atval.
  boost::math::chi_squared chisquaredist(n);
  long double answ=boost::math::quantile(chisquaredist,1-atval);
  //  cout<<n<<" "<<atval<<" "<<answ<<"\n";
  return answ;
  long double *vals=new long double[10000];
  for(unsigned int i=0;i<10000;i++){
    *(vals+i)=0;
    for(unsigned int j=0;j<n;j++){
      long double rnd=rnorm();
      *(vals+i)+=rnd*rnd;
    };
  };
  unsigned int cutoff=(1-atval)*10000;
  //  cout<<"cutoff="<<cutoff<<"\n";
  long double ans=0;
  getpos(10000,vals,1,&cutoff,&ans);
  delete[] vals;
  cout<<"Critical value of chi-squared test is "<<ans<<".\n";
  return ans;
};

extern long double testatval;

void Analysis_mode(int argc,char *argv[]){
  //First try to identify distribution (mandatory arguments)
  unsigned int argno=2;
  Distribution &truedist=read_distribution(argc,argv,argno);


  //Next read adequacy test  (mandatory argument)
  AdequacyTest adequacy_test=read_adequacy_test(argv[argno],truedist);
  if(adequacy_test==NULL){	
    cout<<"Adequacy test \""<<argv[argno]<<"\" not currently available.\n";
    exit(0);
  };

  //Check to see if random seed specified.
  //Check for other options.
  for(argno++;argno<(unsigned int)argc;argno++){
    read_option(argv[argno]);
  };

  if(jackknife){
    adequacy_test=switchtojackknife(adequacy_test);
  };

  if(grouped_data){
    adequacy_test=switchtogrouped(adequacy_test);
  };

  //  cout<<"atval="<<testatval<<"\n";

  //Read data from file
  ifstream datafile(argv[1]);
  if(!datafile.is_open()){
    cerr<<"problem opening file \""<<argv[1]<<"\". Aborting.\n";
    exit(0);
  };


  if(!frequency_table&&!grouped_data){
    long double rd;
    datafile>>rd;
    dblll datlist(truedist.transform_data(rd));
    while(datafile.good()&&!datafile.eof()){
      datafile>>rd;
      if(datafile.good()&&!datafile.eof()){
	datlist.insert(truedist.transform_data(rd));
      };
    };
    
    Data A;
  
    A.n=datlist.length();
    A.data=new long double[A.n];
    datlist.write(A.data);
    
    qsort(A.data,A.n,sizeof(long double),compareLongDouble);    
    
    analyse_data(truedist,adequacy_test,A);
  }else if(frequency_table){
    long double rd;
    unsigned int count;
    datafile>>rd;
    datafile>>count;
    dblctll datlist(truedist.transform_data(rd),count);
    while(datafile.good()&&!datafile.eof()){
      datafile>>rd;
      datafile>>count;
      if(datafile.good()&&!datafile.eof()){
	datlist.insert(truedist.transform_data(rd),count);
      };
    };
    
    FreqTable A;
  
    A.n=datlist.length();
    A.data=new long double[A.n];
    A.count=new unsigned int[A.n];
    long double *tempdata=new long double[A.n];
    unsigned int *tempcount=new unsigned int[A.n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<A.n;i++){
      *(A.count+i)=*(tempcount+A.n-1-i);
      *(A.data+i)=*(tempdata+A.n-1-i);
    };    
    delete[] tempdata;
    delete[] tempcount;
    // Assume frequency table is already sorted
    
    analyse_data_frequency(truedist,adequacy_test,A);
  }else{
    long double rd;
    unsigned int count;
    datafile>>rd;
    char c;
    datafile>>c;
    if(c=='-'){
      datafile>>rd;
    };
    datafile>>count;
    dblctll datlist(truedist.transform_data(rd),count);
    while(datafile.good()&&!datafile.eof()){
      datafile>>rd;
      datafile>>c;
      if(c=='-'){
	datafile>>rd;
      };
      datafile>>count;
      if(datafile.good()&&!datafile.eof()){
	datlist.insert(truedist.transform_data(rd),count);
      };
    };
    
    GroupedData A;
  
    A.n=datlist.length();
    A.data=new long double[A.n];
    A.count=new unsigned int[A.n];
    long double *tempdata=new long double[A.n];
    unsigned int *tempcount=new unsigned int[A.n];
    datlist.write(tempdata,tempcount);
    for(unsigned int i=0;i<A.n;i++){
      *(A.count+i)=*(tempcount+A.n-1-i);
      *(A.data+i)=*(tempdata+A.n-1-i);
    };    
    chisqcval=chisqlookup(A.n-1-truedist.numestpars(),testatval);
    cout<<"Critical value for chi-square test is "<<chisqcval<<".\n";
    delete[] tempdata;
    delete[] tempcount;
    // Assume frequency table is already sorted
    
    analyse_data_grouped(truedist,adequacy_test,A);
    delete[] A.data;
    A.data=NULL;
    delete[] A.count;
    A.count=NULL;
  };
};

typedef long double* (*DistortedSim)(unsigned int n,const Distribution &main);

DistortedSim distort=NULL;
long double contamprob=0;
Distribution *contamDist=NULL;

long double *contamination(unsigned int n,const Distribution &main){
  long double *ans=new long double[n];

  for(unsigned int i=0;i<n;i++){
    if(rand()<contamprob*RAND_MAX){
      if(contamDist==NULL){
	cout<<"Contamination Distribution not Specified. Aborting\n";
	exit(1);
      };
      *(ans+i)=main.transform_data(contamDist->untransform_data(contamDist->random()));
    }else{
      *(ans+i)=main.random();
    };
  };

  return ans;
};


unsigned int sample_bias_step_ncut=0;
long double *sample_bias_step_cuts=NULL;
long double *sample_bias_step_cum_probs=NULL;


long double *sample_bias_step(unsigned int n,const Distribution &main){
  if(sample_bias_step_cum_probs==NULL||sample_bias_step_cuts==NULL){
    cerr<<"Sample bias steps not specified. Aborting.\n";
    exit(1);
  };
  long double *ans=new long double[n];
  //Assume probs sum to 1.
  unsigned int *nsim=new unsigned int[sample_bias_step_ncut+1];
  for(unsigned int i=0;i<=sample_bias_step_ncut;i++){
    *(nsim+i)=0;
  };
  for(unsigned int i=0;i<n;i++){
    long double rnd=((long double)rand())/RAND_MAX;
    unsigned int j=0;
    for(;j<sample_bias_step_ncut&&*(sample_bias_step_cum_probs+j)<rnd;j++);
    (*(nsim+j))++;
  };
  unsigned int *nsimdone=new unsigned int[sample_bias_step_ncut+1];
  for(unsigned int i=0;i<=sample_bias_step_ncut;i++){
    *(nsimdone+i)=0;
  };
  long double **bucket=new long double *[sample_bias_step_ncut+1];
  *bucket=ans;
  for(unsigned int i=0;i<sample_bias_step_ncut;i++){
    *(bucket+i+1)=*(bucket+i)+*(nsim+i);
    cout<<((long double)*(nsim+i))/n<<" ";
  };
  cout<<"\n";
  for(unsigned int i=0;i<n;){
    long double sim=main.random();    
    unsigned int j=0;
    for(;j<sample_bias_step_ncut&&*(sample_bias_step_cuts+j)<sim;j++);    
    if(*(nsimdone+j)<*(nsim+j)){
      i++;
      *(*(bucket+j)+(*(nsimdone+j))++)=sim;
    };
  };

  delete[] bucket;
  delete[] nsim;
  delete[] nsimdone;

  return ans;
};



long double *no_distortion(unsigned int n,const Distribution &main){
  long double *ans=new long double[n];

  for(unsigned int i=0;i<n;i++){
    *(ans+i)=main.random();
  };

  return ans;
};



void Read_distortion(unsigned int &argno,unsigned int argc,char *argv[]){
  if(argno>=argc){
    cout<<"Distortion type not specified.\n";
    exit(0);
  };
  char *distorttype=argv[argno++];
  if(!strcmp(distorttype,"contaminate")){
    if(argno>=argc){
      cout<<"No contamination specified.\n";
      exit(0);
    };
    contamprob=atof(argv[argno++]);
    if(argno>=argc){
      cout<<"Contaminating distribution not specified.\n";
      exit(0);
    };
    contamDist=&(read_distribution(argc,argv,argno));
    distort=contamination;
  }else if(!strcmp(distorttype,"other")){
    //User specified simulation distribution.
    //Treated as 100% contamination.
    if(argno>=argc){
      cout<<"No contamination specified.\n";
      exit(0);
    };
    contamprob=1;
    if(argno>=argc){
      cout<<"Simulation distribution not specified.\n";
      exit(0);
    };
    contamDist=&(read_distribution(argc,argv,argno));
    distort=contamination;
  }else if(!strcmp(distorttype,"no_distortion")){
    distort=no_distortion;
  }else if(!strcmp(distorttype,"sample_bias")){
    if(argno>=argc){
      cout<<"No sample bias type specified.\n";
      exit(0);
    };    
    char *sbtype=argv[argno++];
    if(!strcmp(sbtype,"step")){
      if(argno>=argc){
	cout<<"Sample bias steps not specified.\n";
	exit(0);
      };        
      char *cuts=argv[argno++];  
      if(argno>=argc){
	cout<<"Sample bias cut probabilities not specified.\n";
	exit(0);
      };        
      char *cutprobs=argv[argno++];  
      unsigned int nc=1;
      unsigned int ncp=1;
      for(unsigned int i=0;*(cuts+i);i++){
	if(*(cuts+i)==','){
	  nc++;
	};
      };
      for(unsigned int i=0;*(cutprobs+i);i++){
	if(*(cutprobs+i)==','){
	  ncp++;
	};
      };
      if(ncp!=nc+1){
	cout<<"Number of cuts and proportions do not match.\n";
	exit(0);
      };
      sample_bias_step_ncut=nc;
      sample_bias_step_cuts=new long double[nc];
      sample_bias_step_cum_probs=new long double[nc+1];

      *sample_bias_step_cuts=atof(cuts);
      nc=1;
      for(unsigned int i=0;*(cuts+i+1);i++){
	if(*(cuts+i)==','){
	  *(sample_bias_step_cuts+nc++)=atof(cuts+i+1);
	};
      };
      ncp=1;
      *(sample_bias_step_cum_probs)=atof(cutprobs);
      long double totprob=*(sample_bias_step_cum_probs);
      for(unsigned int i=0;*(cutprobs+i+1);i++){
	if(*(cutprobs+i)==','){
	  totprob+=atof(cutprobs+i+1);
	  *(sample_bias_step_cum_probs+ncp++)=atof(cutprobs+i+1);
	};
      };      
      long double cumprob=*(sample_bias_step_cum_probs);      
      *(sample_bias_step_cum_probs)=cumprob/totprob;
      for(unsigned int j=1;j<=sample_bias_step_ncut;j++){
	cumprob+=*(sample_bias_step_cum_probs+j);      
	*(sample_bias_step_cum_probs+j)=cumprob/totprob;	
      };
      distort=sample_bias_step;
      cout<<"-Inf\t"<<*(sample_bias_step_cuts)<<"\t"<<*(sample_bias_step_cum_probs)<<"\n";
      for(unsigned int j=1;j<sample_bias_step_ncut;j++){
	cout<<*(sample_bias_step_cuts+j-1)<<"\t"<<*(sample_bias_step_cuts+j)<<"\t"<<*(sample_bias_step_cum_probs+j)<<"\n";
      };
      cout<<*(sample_bias_step_cuts+sample_bias_step_ncut-1)<<"\tInf"<<"\t"<<*(sample_bias_step_cum_probs+sample_bias_step_ncut)<<"\n";
    }else{
      cout<<"Sampling bias type \""<<sbtype<<"\" not currently supported.\n";
      exit(0);
    };
  }else{
    cout<<"Distortion type \""<<distorttype<<"\" not currently supported.\n";
    exit(0);
  };
  
};



void Simulation_mode(int argc,char *argv[]){
  if(argc<4){
    cout<<"usage: \'"<<argv[0]<<" FILENAME DISTRIBUTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" simulation SIMULATION_SIZE DISTRIBUTION PARAMETERS DISTORTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" test \'\n";
    exit(0);
  };
  unsigned int simsize=atoi(argv[2]);
  unsigned int argno=3;
  Distribution &truedist=read_distribution(argc,argv,argno);

  Read_distortion(argno,argc,argv);

  //Next read adequacy test  (mandatory argument)
  if(argno>=argc){
    cout<<"Adequacy test not specified.\n";
    exit(0);
  };
  AdequacyTest adequacy_test=read_adequacy_test(argv[argno],truedist);
  if(adequacy_test==NULL){	
    cout<<"Adequacy test \""<<argv[argno]<<"\" not currently available.\n";
    exit(0);
  };


  //Important to read options before simulating, since this allows us
  //to set the random number generator seed before the simulation.
  for(argno++;argno<(unsigned int)argc;argno++){
    read_option(argv[argno]);
  };



  long double *sim=(*distort)(simsize,truedist);

  Data A;
  
  A.n=simsize;
  A.data=new long double[A.n];
  for(unsigned int i=0;i<simsize;i++){
    *(A.data+i)=*(sim+i);
  };

  qsort(A.data,A.n,sizeof(long double),compareLongDouble);    


  if(jackknife){
    adequacy_test=switchtojackknife(adequacy_test);
  };


  if(savedata.is_open()){
    for(unsigned int i=0;i<simsize&&savedata.good();i++){
      savedata<<*(A.data+i)<<"\n";
    };
    savedata.close();
    if(!savedata.good()){
      cout<<"File error in saving data - saved data might be corrupted.\n";
    };
  };

  analyse_data(truedist,adequacy_test,A);

};

void test_mode(int argc,char *argv[]){
  if(argc==2){
    testMLEprob();
  }else if(!strcmp(argv[2],"jackknife")){
    if(argc<5){
      cout<<"Usage: \'"<<argv[0]<<" test jackknife SIZE JACKKNIFESIZE [NUMBER]\'\n";
      exit(0);
    };
    unsigned int n=atoi(argv[3]);
    unsigned int k=atoi(argv[4]);
    unsigned int numtest=100;
    if(argc>5){
      numtest=atoi(argv[5]);
    };
    long double *sample=new long double[n];
    for(unsigned int i=0;i<n;i++){
      *(sample+i)=i;
    };
    long double *to=new long double[k];
    for(unsigned int i=0;i<numtest;i++){
      sortedjackknife(n,k,sample,to);
      for(unsigned int j=0;j<k-1;j++){
	cout<<*(to+j)<<" ";
      };
      cout<<*(to+k-1)<<"\n";
    };
    delete[] sample;
    delete[] to;
  }else if(!strcmp(argv[2],"binomial")){
    if(argc<5){
      cout<<"Usage: \'"<<argv[0]<<" test binomial SIZE probability [NUMBER]\'\n";
      exit(0);
    };
    unsigned int n=atoi(argv[3]);
    long double p=atof(argv[4]);
    unsigned int numtest=100;
    if(argc>5){
      numtest=atoi(argv[5]);
    };
    for(unsigned int i=0;i<numtest;i++){
      cout<<rbinom_fast(n,p)<<"\t";
      if(!((i+1)%10)){
	cout<<"\n";
      };
    };
    cout<<"\n";
  }else if(!strcmp(argv[2],"mle")){
    if(argc<5){
      cout<<"Usage: \'"<<argv[0]<<" test mle datafile distribution [PARAMETERS]\'\n";
      exit(0);
    };


    unsigned int argno=4;
    Distribution &truedist=read_distribution(argc,argv,argno);

    //    cout<<argno<<" "<<argc<<"\n";

    for(;argno<(unsigned int)argc;argno++){
      read_option(argv[argno]);
    };

    ifstream datafile(argv[3]);
    if(!datafile.is_open()){
      cerr<<"problem opening file \""<<argv[3]<<"\". Aborting.\n";
      exit(0);
    };

    cout<<"Test mode.\n";

    if(!frequency_table&&!grouped_data){
      long double rd;
      datafile>>rd;
      dblll datlist(truedist.transform_data(rd));
      while(datafile.good()&&!datafile.eof()){
	datafile>>rd;
	if(datafile.good()&&!datafile.eof()){
	  datlist.insert(truedist.transform_data(rd));
	};
      };
      
      Data A;
    
      A.n=datlist.length();
      A.data=new long double[A.n];
      datlist.write(A.data);
      
      qsort(A.data,A.n,sizeof(long double),compareLongDouble);    
    
      truedist.estimate_data(A.n,A.data);
      truedist.print();
    }else if(frequency_table){
      long double rd;
      unsigned int count;
      datafile>>rd;
      datafile>>count;
      dblctll datlist(truedist.transform_data(rd),count);
      while(datafile.good()&&!datafile.eof()){
	datafile>>rd;
	datafile>>count;
	if(datafile.good()&&!datafile.eof()){
	  datlist.insert(truedist.transform_data(rd),count);
	};
      };
    
      FreqTable A;
  
      A.n=datlist.length();
      A.data=new long double[A.n];
      A.count=new unsigned int[A.n];
      long double *tempdata=new long double[A.n];
      unsigned int *tempcount=new unsigned int[A.n];
      datlist.write(tempdata,tempcount);
      for(unsigned int i=0;i<A.n;i++){
	*(A.count+i)=*(tempcount+A.n-1-i);
	*(A.data+i)=*(tempdata+A.n-1-i);
      };    
      delete[] tempdata;
      delete[] tempcount;
      // Assume frequency table is already sorted
      //      truedist.estimate_data(A.n,A.data);
    

    }else{
      cout<<"Test Mode, grouped data.\n";
      long double rd;
      unsigned int count;
      datafile>>rd;
      char c;
      datafile>>c;
      if(c=='-'){
	datafile>>rd;
      };
      datafile>>count;
      dblctll datlist(truedist.transform_data(rd),count);
      while(datafile.good()&&!datafile.eof()){
	datafile>>rd;
	datafile>>c;
	if(c=='-'){
	  datafile>>rd;
	};
	datafile>>count;
	if(datafile.good()&&!datafile.eof()){
	  datlist.insert(truedist.transform_data(rd),count);
	};
      };
      
      GroupedData A;
      
      A.n=datlist.length();
      A.data=new long double[A.n];
      A.count=new unsigned int[A.n];
      long double *tempdata=new long double[A.n];
      unsigned int *tempcount=new unsigned int[A.n];
      datlist.write(tempdata,tempcount);
      for(unsigned int i=0;i<A.n;i++){
	*(A.count+i)=*(tempcount+A.n-1-i);
	*(A.data+i)=*(tempdata+A.n-1-i);
      };    
      delete[] tempdata;
      delete[] tempcount;

      unsigned int tot=0;
      for(unsigned int i=0;i<A.n;i++){
	tot+=*(A.count+i);
	cout<<(i?*(A.data+i-1):0)<<"-"<<*(A.data+i)<<": "<<*(A.count+i)<<"\n";
      };
      cout<<tot<<"\n";

      truedist.estimate_data(A.n,A.count,A.data);
      truedist.print();
    
    };

  };
};


int main(int argc,char* argv[]){
  set_up_distributions();
  if(argc==1){
    cout<<"usage: \'"<<argv[0]<<" FILENAME DISTRIBUTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" simulation NUMBER DISTRIBUTION PARAMETERS DISTORTION PARAMETERS ADEQUACY_TEST [OPTIONS]\n";
    cout<<"or: \'"<<argv[0]<<" test \'\n";
    exit(0);
  };
  if(!strcmp(argv[1],"test")){
    test_mode(argc,argv);
  }else if(!strcmp(argv[1],"simulation")){
    Simulation_mode(argc,argv);
  }else{    
    //data mode
    Analysis_mode(argc,argv);
  };
};



